#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <strsafe.h>
#include <Shlwapi.h>

#define MSODLL TEXT("mso.dll")
#define MSODLLSIZE (ARRAYSIZE(MSODLL) - 1)

#pragma function(memset)
void * __cdecl memset (void *dst, int val, size_t count)
{
    void *start = dst;

    while (count--) {
            *(char *)dst = (char)val;
            dst = (char *)dst + 1;
    }

    return(start);
}

void ShowErrorMessage(DWORD ErrorCode)
{
    LPTSTR pErrorMessage;

    DWORD Chars = FormatMessage(
        FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        ErrorCode,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR)&pErrorMessage,
        0,
        NULL);

    if (Chars != 0)
    {
        MessageBox(0, pErrorMessage, TEXT("unOFFIC Error"), MB_ICONERROR);
        LocalFree(pErrorMessage);
    }
    else
    {
        MessageBox(0, TEXT("Operation failed"), TEXT("unOFFIC Error"), MB_ICONERROR);
    }
}

BYTE const * FindMem(BYTE const *pBuffer, size_t cbBuffer, BYTE const *pSubstring, size_t cbSubstring)
{  
    BYTE const *cp = pBuffer;
    BYTE const *p1, *p2;
  
    if (!cbSubstring)
        return pBuffer;
  
    while (cp < pBuffer + cbBuffer)
    {
        p1 = cp;
        p2 = pSubstring;
  
        while ( p1 < pBuffer + cbBuffer && p2 < pSubstring + cbSubstring && !(*p1 - *p2))
            p1++, p2++;
  
        if (p2 == pSubstring + cbSubstring)
            return cp;
  
        cp++;
    }
  
    return NULL;
}

DWORD GetModuleImageSize(HANDLE hProcess, LPVOID lpBaseOfDll)
{
    MEMORY_BASIC_INFORMATION Mbi;
    LPVOID Mem = NULL;
    DWORD Size = 0;

    if (VirtualQueryEx(hProcess, lpBaseOfDll, &Mbi, sizeof(Mbi)))
    {
        Mem = VirtualAlloc(NULL, Mbi.RegionSize, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
        if (Mem)
        {
            if (ReadProcessMemory(hProcess, lpBaseOfDll, Mem, Mbi.RegionSize, NULL))
            {
                IMAGE_DOS_HEADER *DosHeader = (IMAGE_DOS_HEADER*)Mem;
                IMAGE_NT_HEADERS *NtHeaders = (IMAGE_NT_HEADERS*)((BYTE*)Mem + DosHeader->e_lfanew);
                Size = NtHeaders->OptionalHeader.SizeOfImage;
            }
            VirtualFree(Mem, 0, MEM_RELEASE);
        }
    }
    
    return Size;
}

void main()
{
    TCHAR *pPathName;
    TCHAR pModuleName[MAX_PATH];
    STARTUPINFO StartupInfo = { 0 };
    PROCESS_INFORMATION ProcessInformation = { 0 };
    DEBUG_EVENT DebugEvent;
    DWORD Length;
    
    LPVOID Mem = NULL;
    const BYTE Pattern[] = { 0x68, 0x00, 0x02, 0x00, 0x01, 0xFF };
    const BYTE Replacement[] = { 0x68, 0x00, 0x00, 0x00, 0x01, 0xFF };
    const BYTE *Found;
   
    pPathName = GetCommandLine();
    if (*pPathName == '"')
    {
        pPathName = StrChr(pPathName + 1, '"');
        if (!pPathName)
            goto Exit;
        pPathName++;
    }
    pPathName = StrChr(pPathName, ' ');
    if (!pPathName)
        goto Exit;
    while (*pPathName == ' ' || *pPathName == '\t') pPathName++;

    StartupInfo.cb = sizeof(StartupInfo);
    if (!CreateProcess(NULL, pPathName, NULL, NULL, FALSE, DEBUG_PROCESS | DEBUG_ONLY_THIS_PROCESS, NULL, NULL, &StartupInfo, &ProcessInformation))
    {
        // Try launching again without debugging. Maybe this is wrong bitness. We can't to much, but at least the process will start.
        if (!CreateProcess(NULL, pPathName, NULL, NULL, FALSE, 0, NULL, NULL, &StartupInfo, &ProcessInformation))
            goto ReportErrorAndExit;

        // If this succeeded, just exit...
        goto Exit;
    }

    DebugSetProcessKillOnExit(FALSE);

    while (WaitForDebugEvent(&DebugEvent, INFINITE))
    {
        switch (DebugEvent.dwDebugEventCode)
        {
        case CREATE_PROCESS_DEBUG_EVENT:
            CloseHandle(DebugEvent.u.CreateProcessInfo.hFile);
            break;
        case LOAD_DLL_DEBUG_EVENT:
            Length = GetFinalPathNameByHandle(DebugEvent.u.LoadDll.hFile, pModuleName, ARRAYSIZE(pModuleName), 0);
            if (Length > MSODLLSIZE && Length < ARRAYSIZE(pModuleName) &&
                CompareStringEx(LOCALE_NAME_INVARIANT, NORM_IGNORECASE, &pModuleName[Length - MSODLLSIZE], MSODLLSIZE, MSODLL, MSODLLSIZE, NULL, NULL, 0) == CSTR_EQUAL)
            {
                DWORD ModuleSize = GetModuleImageSize(ProcessInformation.hProcess, DebugEvent.u.LoadDll.lpBaseOfDll);
                if (ModuleSize)
                {
                    Mem = VirtualAlloc(NULL, ModuleSize, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
                    if (Mem == NULL)
                        goto ReportErrorAndExit;

                    if (!ReadProcessMemory(ProcessInformation.hProcess, DebugEvent.u.LoadDll.lpBaseOfDll, Mem, ModuleSize, NULL))
                        goto ReportErrorAndExit;
                    
                    Found = FindMem((BYTE*)Mem, ModuleSize, Pattern, sizeof(Pattern));

                    if (Found)
                    {
                        DWORD Offset = Found - (BYTE*)Mem;
                        if (!WriteProcessMemory(ProcessInformation.hProcess, (BYTE*)DebugEvent.u.LoadDll.lpBaseOfDll + Offset, Replacement, sizeof(Replacement), NULL))
                            goto ReportErrorAndExit;

                        goto Exit;
                    }
                    else
                    {
                        SetLastError(ERROR_NOT_SUPPORTED);
                        goto ReportErrorAndExit;
                    }
                }
                goto ReportErrorAndExit;
            }
            CloseHandle(DebugEvent.u.LoadDll.hFile);
            break;
        case EXIT_PROCESS_DEBUG_EVENT:
            goto Exit;
        }

        if (!ContinueDebugEvent(DebugEvent.dwProcessId, DebugEvent.dwThreadId, DBG_EXCEPTION_NOT_HANDLED ))
        {
            break;
        }
    }

ReportErrorAndExit:
    ShowErrorMessage(GetLastError());

Exit:
    if (Mem != NULL)
        VirtualFree(Mem, 0, MEM_RELEASE);

    if (ProcessInformation.hProcess)
    {
        CloseHandle(ProcessInformation.hProcess);
        CloseHandle(ProcessInformation.hThread);
    }
}
